from __future__ import absolute_import, division, print_function

__all__ = [
    "__title__", "__summary__", "__uri__", "__version__", "__author__",
    "__email__", "__license__", "__copyright__",
]

__title__ = "gql-checker"
__summary__ = (
    "Flake8 and pylama plugin that checks gql GraphQL calls."
)
__uri__ = "https://github.com/graphql-python/gql-checker"

__version__ = "0.1"

__author__ = "Syrus Akbary"
__email__ = "me@syrusakbary.com"

__license__ = "MIT"
__copyright__ = "Copyright 2016 %s" % __author__
